@@ -415,6 +415,22 @@ eog_image_real_transform (EogImage     *img,
 	}
 }
 
+static gboolean
+do_emit_size_prepared_signal (EogImage *img)
+{
+	g_signal_emit (img, signals[SIGNAL_SIZE_PREPARED], 0,
+		       img->priv->width, img->priv->height);
+	return FALSE;
+}
+
+static void
+eog_image_emit_size_prepared (EogImage *img)
+{
+	gdk_threads_add_idle_full (G_PRIORITY_DEFAULT_IDLE,
+				   (GSourceFunc) do_emit_size_prepared_signal,
+				   g_object_ref (img), g_object_unref);
+}
+
 static gboolean
 check_loader_threadsafety (GdkPixbufLoader *loader, gboolean *result)
 {
@@ -475,7 +491,7 @@ eog_image_size_prepared (GdkPixbufLoader *loader,
 #else
 	if (img->priv->threadsafe_format)
 #endif
-		g_signal_emit (img, signals[SIGNAL_SIZE_PREPARED], 0, width, height);
+		eog_image_emit_size_prepared (img);
 }
 
 static EogMetadataReader*
@@ -662,15 +678,18 @@ static void
 eog_image_set_orientation (EogImage *img)
 {
 	EogImagePrivate *priv;
+	ExifData* exif;
 
 	g_return_if_fail (EOG_IS_IMAGE (img));
 
 	priv = img->priv;
 
-	if (priv->exif != NULL) {
-		ExifByteOrder o = exif_data_get_byte_order (priv->exif);
+	exif = (ExifData*) eog_image_get_exif_info (img);
 
-		ExifEntry *entry = exif_data_get_entry (priv->exif,
+	if (exif != NULL) {
+		ExifByteOrder o = exif_data_get_byte_order (exif);
+
+		ExifEntry *entry = exif_data_get_entry (exif,
 							EXIF_TAG_ORIENTATION);
 
 		if (entry && entry->data != NULL) {
@@ -678,6 +697,9 @@ eog_image_set_orientation (EogImage *img)
 		}
 	}
 
+	/* exif_data_unref handles NULL values like g_free */
+	exif_data_unref (exif);
+
 	if (priv->orientation > 4 &&
 	    priv->orientation < 9) {
 		gint tmp;
@@ -771,11 +793,7 @@ eog_image_set_exif_data (EogImage *img, EogMetadataReader *md_reader)
 		/* Emit size prepared signal if we have the size */
 		if (priv->width > 0 &&
 		    priv->height > 0) {
-			g_signal_emit (img,
-				       signals[SIGNAL_SIZE_PREPARED],
-				       0,
-				       priv->width,
-				       priv->height);
+			eog_image_emit_size_prepared (img);
 		}
 	}
 #else
@@ -827,7 +845,8 @@ eog_image_real_load (EogImage *img,
 	gboolean set_metadata = TRUE;
 	gboolean read_image_data = (data2read & EOG_IMAGE_DATA_IMAGE);
 	gboolean read_only_dimension = (data2read & EOG_IMAGE_DATA_DIMENSION) &&
-					!read_image_data;
+				  ((data2read ^ EOG_IMAGE_DATA_DIMENSION) == 0);
+
 
 	priv = img->priv;
 
@@ -961,11 +980,8 @@ eog_image_real_load (EogImage *img,
 				}
 
 				if (priv->threadsafe_format)
-					g_signal_emit (img,
-						       signals[SIGNAL_SIZE_PREPARED],
-						       0,
-						       priv->width,
-						       priv->height);
+					eog_image_emit_size_prepared (img);
+
                                 priv->metadata_status = EOG_IMAGE_METADATA_NOT_AVAILABLE;
                         }
 
@@ -1051,11 +1067,7 @@ eog_image_real_load (EogImage *img,
 			/* If it's non-threadsafe loader, then trigger window
  			 * showing in the end of the process. */
 			if (!priv->threadsafe_format)
-				g_signal_emit (img,
-					       signals[SIGNAL_SIZE_PREPARED],
-					       0,
-					       priv->width,
-					       priv->height);
+				eog_image_emit_size_prepared (img);
 		} else {
 			/* Some loaders don't report errors correctly.
 			 * Error will be set below. */
@@ -1153,7 +1165,10 @@ eog_image_load (EogImage *img, EogImageData data2read, EogJob *job, GError **err
 	success = eog_image_real_load (img, data2read, job, error);
 
 #ifdef HAVE_EXIF
-	if (priv->autorotate) {
+	/* Check that the metadata was loaded at least once before
+	 * trying to autorotate. */
+	if (priv->autorotate && 
+	    priv->metadata_status == EOG_IMAGE_METADATA_READY) {
 		eog_image_real_autorotate (img);
 	}
 #endif
